@@ -23,15 +23,19 @@
 
 #include <glib-object.h>
 #include "config.h"
+#include "as-utils-private.h"
 
 G_BEGIN_DECLS
+#pragma GCC visibility push(hidden)
 
 #define AS_CONFIG_NAME "/etc/appstream.conf"
 #define AS_APPSTREAM_CACHE_PATH "/var/cache/app-info"
 
 /* declared in as-data-pool.c */
+AS_INTERNAL_VISIBLE
 extern const gchar *AS_APPSTREAM_METADATA_PATHS[4];
 
+#pragma GCC visibility pop
 G_END_DECLS
 
 #endif /* __AS_SETTINGSPRIVATE_H */
