@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2012-2016 Matthias Klumpp <matthias@tenstral.net>
- * Copyright (C)      2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C)      2016 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -38,6 +38,7 @@
 #include <errno.h>
 
 #include "as-category.h"
+#include "as-resources.h"
 
 /**
  * SECTION:as-utils
@@ -490,9 +491,9 @@ as_ptr_array_to_strv (GPtrArray *array)
 	g_return_val_if_fail (array != NULL, NULL);
 
 	/* copy the array to a strv */
-	value = g_new0 (gchar *, array->len + 1);
-	for (i=0; i<array->len; i++) {
-		value_temp = (const gchar *) g_ptr_array_index (array, i);
+	value = g_new0 (gchar*, array->len + 1);
+	for (i = 0; i < array->len; i++) {
+		value_temp = (const gchar*) g_ptr_array_index (array, i);
 		value[i] = g_strdup (value_temp);
 	}
 
@@ -694,3 +695,204 @@ as_arch_compatible (const gchar *arch1, const gchar *arch2)
 		return TRUE;
 	return FALSE;
 }
+
+/**
+ * as_utils_locale_to_language:
+ * @locale: The locale string.
+ *
+ * Get language part from a locale string.
+ */
+gchar*
+as_utils_locale_to_language (const gchar *locale)
+{
+	gchar *tmp;
+	gchar *country_code;
+
+	/* invalid */
+	if (locale == NULL)
+		return NULL;
+
+	/* return the part before the _ (not always 2 chars!) */
+	country_code = g_strdup (locale);
+	tmp = g_strstr_len (country_code, -1, "_");
+	if (tmp != NULL)
+		*tmp = '\0';
+	return country_code;
+}
+
+/**
+ * as_ptr_array_find_string:
+ * @array: gchar* array
+ * @value: string to find
+ *
+ * Finds a string in a pointer array.
+ *
+ * Returns: the const string, or %NULL if not found
+ **/
+const gchar*
+as_ptr_array_find_string (GPtrArray *array, const gchar *value)
+{
+	const gchar *tmp;
+	guint i;
+	for (i = 0; i < array->len; i++) {
+		tmp = g_ptr_array_index (array, i);
+		if (g_strcmp0 (tmp, value) == 0)
+			return tmp;
+	}
+	return NULL;
+}
+
+/**
+ * as_utils_locale_is_compatible:
+ * @locale1: a locale string, or %NULL
+ * @locale2: a locale string, or %NULL
+ *
+ * Calculates if one locale is compatible with another.
+ * When doing the calculation the locale and language code is taken into
+ * account if possible.
+ *
+ * Returns: %TRUE if the locale is compatible.
+ *
+ * Since: 0.9.5
+ **/
+gboolean
+as_utils_locale_is_compatible (const gchar *locale1, const gchar *locale2)
+{
+	g_autofree gchar *lang1 = as_utils_locale_to_language (locale1);
+	g_autofree gchar *lang2 = as_utils_locale_to_language (locale2);
+
+	/* we've specified "don't care" and locale unspecified */
+	if (locale1 == NULL && locale2 == NULL)
+		return TRUE;
+
+	/* forward */
+	if (locale1 == NULL && locale2 != NULL) {
+		const gchar *const *locales = g_get_language_names ();
+		return g_strv_contains (locales, locale2) ||
+		       g_strv_contains (locales, lang2);
+	}
+
+	/* backwards */
+	if (locale1 != NULL && locale2 == NULL) {
+		const gchar *const *locales = g_get_language_names ();
+		return g_strv_contains (locales, locale1) ||
+		       g_strv_contains (locales, lang1);
+	}
+
+	/* both specified */
+	if (g_strcmp0 (locale1, locale2) == 0)
+		return TRUE;
+	if (g_strcmp0 (locale1, lang2) == 0)
+		return TRUE;
+	if (g_strcmp0 (lang1, locale2) == 0)
+		return TRUE;
+	return FALSE;
+}
+
+/**
+ * as_utils_search_token_valid:
+ * @token: the search token
+ *
+ * Checks the search token if it is valid. Valid tokens are at least 3 chars in
+ * length, not common words like "and", and do not contain markup.
+ *
+ * Returns: %TRUE is the search token was valid.
+ **/
+gboolean
+as_utils_search_token_valid (const gchar *token)
+{
+	guint i;
+	/* TODO: Localize this list */
+	const gchar *blacklist[] = {
+		"and", "the", "application", "for", "you", "your",
+		"with", "can", "are", "from", "that", "use", "allows", "also",
+		"this", "other", "all", "using", "has", "some", "like", "them",
+		"well", "not", "using", "not", "but", "set", "its", "into",
+		"such", "was", "they", "where", "want", "only", "about",
+		"uses", "font", "features", "designed", "provides", "which",
+		"many", "used", "org", "fonts", "open", "more", "based",
+		"different", "including", "will", "multiple", "out", "have",
+		"each", "when", "need", "most", "both", "their", "even",
+		"way", "several", "been", "while", "very", "add", "under",
+		"what", "those", "much", "either", "currently", "one",
+		"support", "make", "over", "these", "there", "without", "etc",
+		"main",
+		NULL };
+	if (strlen (token) < 3)
+		return FALSE;
+	if (g_strstr_len (token, -1, "<") != NULL)
+		return FALSE;
+	if (g_strstr_len (token, -1, ">") != NULL)
+		return FALSE;
+	if (g_strstr_len (token, -1, "(") != NULL)
+		return FALSE;
+	if (g_strstr_len (token, -1, ")") != NULL)
+		return FALSE;
+	for (i = 0; blacklist[i] != NULL; i++)  {
+		if (g_strcmp0 (token, blacklist[i]) == 0)
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
+/**
+ * as_utils_is_category_id:
+ * @category_name: an XDG category name, e.g. "ProjectManagement"
+ *
+ * Searches the known list of registered XDG category names.
+ * See https://specifications.freedesktop.org/menu-spec/menu-spec-1.0.html#category-registry
+ * for a reference.
+ *
+ * Returns: %TRUE if the category name is valid
+ *
+ * Since: 0.9.7
+ **/
+gboolean
+as_utils_is_category_name (const gchar *category_name)
+{
+	g_autoptr(GBytes) data = NULL;
+	g_autofree gchar *key = NULL;
+
+	/* custom spec-extensions are generally valid if prefixed correctly */
+	if (g_str_has_prefix (category_name, "X-"))
+		return TRUE;
+
+	/* load the readonly data section and look for the category name */
+	data = g_resource_lookup_data (as_get_resource (),
+				       "/org/freedesktop/appstream/xdg-category-names.txt",
+				       G_RESOURCE_LOOKUP_FLAGS_NONE,
+				       NULL);
+	if (data == NULL)
+		return FALSE;
+	key = g_strdup_printf ("\n%s\n", category_name);
+	return g_strstr_len (g_bytes_get_data (data, NULL), -1, key) != NULL;
+}
+
+/**
+ * as_utils_is_tld:
+ * @tld: a top-level domain without dot, e.g. "de", "org", "name"
+ *
+ * Searches the known list of TLDs we allow for AppStream IDs.
+ * This excludes internationalized names.
+ *
+ * Returns: %TRUE if the TLD is valid
+ *
+ * Since: 0.9.8
+ **/
+gboolean
+as_utils_is_tld (const gchar *tld)
+{
+	g_autoptr(GBytes) data = NULL;
+	g_autofree gchar *key = NULL;
+
+	/* load the readonly data section and look for the TLD */
+	data = g_resource_lookup_data (as_get_resource (),
+				       "/org/freedesktop/appstream/iana-filtered-tld-list.txt",
+				       G_RESOURCE_LOOKUP_FLAGS_NONE,
+				       NULL);
+	if (data == NULL)
+		return FALSE;
+	key = g_strdup_printf ("\n%s\n", tld);
+	return g_strstr_len (g_bytes_get_data (data, NULL), -1, key) != NULL;
+}
