@@ -29,16 +29,18 @@
 #include "as-metadata.h"
 #include "as-component-private.h"
 #include "as-screenshot-private.h"
+#include "as-release-private.h"
 
 typedef struct
 {
 	gchar *locale;
-	gchar *locale_short;
 	gchar *origin;
 	gchar *media_baseurl;
 
 	gchar *arch;
 	gint default_priority;
+	AsParserMode mode;
+	gboolean check_valid;
 } AsYAMLDataPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsYAMLData, as_yamldata, G_TYPE_OBJECT)
@@ -66,6 +68,10 @@ as_yamldata_init (AsYAMLData *ydt)
 	priv->origin = NULL;
 	priv->media_baseurl = NULL;
 	priv->default_priority = 0;
+	priv->check_valid = TRUE;
+
+	/* the YAML data is only for distro-metadata at time */
+	priv->mode = AS_PARSER_MODE_DISTRO;
 }
 
 /**
@@ -78,7 +84,6 @@ as_yamldata_finalize (GObject *object)
 	AsYAMLDataPrivate *priv = GET_PRIVATE (ydt);
 
 	g_free (priv->locale);
-	g_free (priv->locale_short);
 	g_free (priv->origin);
 	g_free (priv->media_baseurl);
 	g_free (priv->arch);
@@ -95,16 +100,11 @@ as_yamldata_finalize (GObject *object)
 void
 as_yamldata_initialize (AsYAMLData *ydt, const gchar *locale, const gchar *origin, const gchar *media_baseurl, const gchar *arch, gint priority)
 {
-	g_auto(GStrv) strv = NULL;
 	AsYAMLDataPrivate *priv = GET_PRIVATE (ydt);
 
 	g_free (priv->locale);
-	g_free (priv->locale_short);
 	priv->locale = g_strdup (locale);
 
-	strv = g_strsplit (priv->locale, "_", 0);
-	priv->locale_short = g_strdup (strv[0]);
-
 	g_free (priv->origin);
 	priv->origin = g_strdup (origin);
 
@@ -118,19 +118,19 @@ as_yamldata_initialize (AsYAMLData *ydt, const gchar *locale, const gchar *origi
 }
 
 /**
- * dep11_print_unknown:
+ * as_yaml_print_unknown:
  */
 static void
-dep11_print_unknown (const gchar *root, const gchar *key)
+as_yaml_print_unknown (const gchar *root, const gchar *key)
 {
-	g_debug ("DEP11: Unknown key '%s/%s' found.", root, key);
+	g_debug ("YAML: Unknown field '%s/%s' found.", root, key);
 }
 
 /**
- * as_yamldata_free_node:
+ * as_yaml_free_node:
  */
 static gboolean
-as_yamldata_free_node (GNode *node, gpointer data)
+as_yaml_free_node (GNode *node, gpointer data)
 {
 	if (node->data != NULL)
 		g_free (node->data);
@@ -139,22 +139,54 @@ as_yamldata_free_node (GNode *node, gpointer data)
 }
 
 /**
- * dep11_yaml_process_layer:
+ * as_yaml_node_get_key:
+ *
+ * Helper method to get the key of a node.
+ */
+const gchar*
+as_yaml_node_get_key (GNode *n)
+{
+	return (const gchar*) n->data;
+}
+
+/**
+ * as_yaml_node_get_value:
+ *
+ * Helper method to get the value of a node.
+ */
+const gchar*
+as_yaml_node_get_value (GNode *n)
+{
+	if (n->children)
+		return (const gchar*) n->children->data;
+	else
+		return NULL;
+}
+
+/**
+ * as_yaml_process_layer:
  *
  * Create GNode tree from DEP-11 YAML document
  */
 static void
-dep11_yaml_process_layer (yaml_parser_t *parser, GNode *data)
+as_yaml_process_layer (yaml_parser_t *parser, GNode *data, GError **error)
 {
 	GNode *last_leaf = data;
 	GNode *last_scalar;
 	yaml_event_t event;
 	gboolean parse = TRUE;
 	gboolean in_sequence = FALSE;
+	GError *tmp_error = NULL;
 	int storage = YAML_VAR; /* the first element must always be of type VAR */
 
 	while (parse) {
-		yaml_parser_parse (parser, &event);
+		if (!yaml_parser_parse (parser, &event)) {
+			g_set_error (error,
+					AS_METADATA_ERROR,
+					AS_METADATA_ERROR_PARSE,
+					"Invalid DEP-11 file found. Could not parse YAML: %s", parser->problem);
+			break;
+		}
 
 		/* Parse value either as a new leaf in the mapping
 		 * or as a leaf value (one of them, in case it's a sequence) */
@@ -179,7 +211,13 @@ dep11_yaml_process_layer (yaml_parser_t *parser, GNode *data)
 				last_scalar = last_leaf;
 				if (in_sequence)
 					last_leaf = g_node_append (last_leaf, g_node_new (g_strdup ("-")));
-				dep11_yaml_process_layer (parser, last_leaf);
+
+				as_yaml_process_layer (parser, last_leaf, &tmp_error);
+				if (tmp_error != NULL) {
+					g_propagate_error (error, tmp_error);
+					parse = FALSE;
+				}
+
 				last_leaf = last_scalar;
 				storage ^= YAML_VAL; /* Flip VAR/VAL, without touching SEQ */
 				break;
@@ -205,37 +243,31 @@ as_yamldata_get_localized_node (AsYAMLData *ydt, GNode *node, gchar *locale_over
 {
 	GNode *n;
 	GNode *tnode = NULL;
-	gchar *key;
+	const gchar *key;
 	const gchar *locale;
-	const gchar *locale_short = NULL;
 	AsYAMLDataPrivate *priv = GET_PRIVATE (ydt);
 
-	if (locale_override == NULL) {
+	if (locale_override == NULL)
 		locale = priv->locale;
-		locale_short = priv->locale_short;
-	} else {
+	else
 		locale = locale_override;
-		locale_short = NULL;
-	}
 
 	for (n = node->children; n != NULL; n = n->next) {
-		key = (gchar*) n->data;
+		key = as_yaml_node_get_key (n);
 
 		if ((tnode == NULL) && (g_strcmp0 (key, "C") == 0)) {
 			tnode = n;
-			if (locale == NULL)
+			if (g_strcmp0 (locale, "C") == 0)
 				goto out;
 		}
 
-		if (g_strcmp0 (key, locale) == 0) {
+		if (g_strcmp0 (locale, key) == 0) {
 			tnode = n;
 			goto out;
 		}
 
-		if (g_strcmp0 (key, locale_short) == 0) {
+		if (as_utils_locale_is_compatible (locale, key))
 			tnode = n;
-			goto out;
-		}
 	}
 
 out:
@@ -260,15 +292,15 @@ as_yamldata_get_localized_value (AsYAMLData *ydt, GNode *node, gchar *locale_ove
 }
 
 /**
- * dep11_list_to_string_array:
+ * as_yaml_list_to_string_array:
  */
 static void
-dep11_list_to_string_array (GNode *node, GPtrArray *array)
+as_yaml_list_to_string_array (GNode *node, GPtrArray *array)
 {
 	GNode *n;
 
 	for (n = node->children; n != NULL; n = n->next) {
-		g_ptr_array_add (array, g_strdup ((gchar*) n->data));
+		g_ptr_array_add (array, g_strdup (as_yaml_node_get_key (n)));
 	}
 }
 
@@ -291,7 +323,7 @@ as_yamldata_process_keywords (AsYAMLData *ydt, GNode *node, AsComponent *cpt)
 	if (tnode == NULL)
 		return;
 
-	dep11_list_to_string_array (tnode, keywords);
+	as_yaml_list_to_string_array (tnode, keywords);
 
 	strv = as_ptr_array_to_strv (keywords);
 	as_component_set_keywords (cpt, strv, NULL);
@@ -306,17 +338,18 @@ static void
 dep11_process_urls (GNode *node, AsComponent *cpt)
 {
 	GNode *n;
-	gchar *key;
-	gchar *value;
 	AsUrlKind url_kind;
 
 	for (n = node->children; n != NULL; n = n->next) {
-			key = (gchar*) n->data;
-			value = (gchar*) n->children->data;
+		const gchar *key;
+		const gchar *value;
 
-			url_kind = as_url_kind_from_string (key);
-			if ((url_kind != AS_URL_KIND_UNKNOWN) && (value != NULL))
-				as_component_add_url (cpt, url_kind, value);
+		key = as_yaml_node_get_key (n);
+		value = as_yaml_node_get_value (n);
+
+		url_kind = as_url_kind_from_string (key);
+		if ((url_kind != AS_URL_KIND_UNKNOWN) && (value != NULL))
+			as_component_add_url (cpt, url_kind, value);
 	}
 }
 
@@ -327,8 +360,6 @@ static void
 as_yamldata_process_icon (AsYAMLData *ydt, GNode *node, AsComponent *cpt, AsIconKind kind)
 {
 	GNode *n;
-	gchar *key;
-	gchar *value;
 	guint64 size;
 	g_autoptr(AsIcon) icon = NULL;
 	AsYAMLDataPrivate *priv = GET_PRIVATE (ydt);
@@ -337,8 +368,11 @@ as_yamldata_process_icon (AsYAMLData *ydt, GNode *node, AsComponent *cpt, AsIcon
 	as_icon_set_kind (icon, kind);
 
 	for (n = node->children; n != NULL; n = n->next) {
-		key = (gchar*) n->data;
-		value = (gchar*) n->children->data;
+		const gchar *key;
+		const gchar *value;
+
+		key = as_yaml_node_get_key (n);
+		value = as_yaml_node_get_value (n);
 
 		if (g_strcmp0 (key, "width") == 0) {
 			size = g_ascii_strtoll (value, NULL, 10);
@@ -354,10 +388,9 @@ as_yamldata_process_icon (AsYAMLData *ydt, GNode *node, AsComponent *cpt, AsIcon
 						as_icon_set_url (icon, value);
 					} else {
 						/* handle the media baseurl */
-						gchar *tmp;
-						tmp = g_build_filename (priv->media_baseurl, value, NULL);
-						as_icon_set_url (icon, tmp);
-						g_free (tmp);
+						g_autofree gchar *url = NULL;
+						url = g_build_filename (priv->media_baseurl, value, NULL);
+						as_icon_set_url (icon, url);
 					}
 				}
 			} else {
@@ -378,12 +411,12 @@ static void
 as_yamldata_process_icons (AsYAMLData *ydt, GNode *node, AsComponent *cpt)
 {
 	GNode *n;
-	gchar *key;
-	gchar *value;
+	const gchar *key;
+	const gchar *value;
 
 	for (n = node->children; n != NULL; n = n->next) {
-		key = (gchar*) n->data;
-		value = (gchar*) n->children->data;
+		key = (const gchar*) n->data;
+		value = (const gchar*) n->children->data;
 
 		if (g_strcmp0 (key, "stock") == 0) {
 			g_autoptr(AsIcon) icon = as_icon_new ();
@@ -423,10 +456,9 @@ dep11_process_provides (GNode *node, AsComponent *cpt)
 {
 	GNode *n;
 	GNode *sn;
-	gchar *key;
 
 	for (n = node->children; n != NULL; n = n->next) {
-		key = (gchar*) n->data;
+		const gchar *key = as_yaml_node_get_key (n);
 
 		if (g_strcmp0 (key, "libraries") == 0) {
 			for (sn = n->children; sn != NULL; sn = sn->next) {
@@ -531,14 +563,13 @@ dep11_process_image (AsYAMLData *ydt, GNode *node, AsScreenshot *scr)
 	img = as_image_new ();
 
 	for (n = node->children; n != NULL; n = n->next) {
-		gchar *key;
-		gchar *value;
+		const gchar *key;
+		const gchar *value;
 		guint64 size;
 
-		key = (gchar*) n->data;
-		if (n->children)
-			value = (gchar*) n->children->data;
-		else
+		key = as_yaml_node_get_key (n);
+		value = as_yaml_node_get_value (n);
+		if (value == NULL)
 			continue; /* there should be no key without value */
 
 		if (g_strcmp0 (key, "width") == 0) {
@@ -558,8 +589,10 @@ dep11_process_image (AsYAMLData *ydt, GNode *node, AsScreenshot *scr)
 				as_image_set_url (img, tmp);
 				g_free (tmp);
 			}
+		} else if (g_strcmp0 (key, "lang") == 0) {
+			as_image_set_locale (img, value);
 		} else {
-			dep11_print_unknown ("image", key);
+			as_yaml_print_unknown ("image", key);
 		}
 	}
 
@@ -585,14 +618,11 @@ as_yamldata_process_screenshots (AsYAMLData *ydt, GNode *node, AsComponent *cpt)
 
 		for (n = sn->children; n != NULL; n = n->next) {
 			GNode *in;
-			gchar *key;
-			gchar *value;
+			const gchar *key;
+			const gchar *value;
 
-			key = (gchar*) n->data;
-			if (n->children)
-				value = (gchar*) n->children->data;
-			else
-				value = NULL;
+			key = as_yaml_node_get_key (n);
+			value = as_yaml_node_get_value (n);
 
 			if (g_strcmp0 (key, "default") == 0) {
 				if (g_strcmp0 (value, "yes") == 0)
@@ -613,7 +643,7 @@ as_yamldata_process_screenshots (AsYAMLData *ydt, GNode *node, AsComponent *cpt)
 					dep11_process_image (ydt, in, scr);
 				}
 			} else {
-				dep11_print_unknown ("screenshot", key);
+				as_yaml_print_unknown ("screenshot", key);
 			}
 		}
 
@@ -642,14 +672,11 @@ as_yamldata_process_releases (AsYAMLData *ydt, GNode *node, AsComponent *cpt)
 		as_release_set_active_locale (rel, as_component_get_active_locale (cpt));
 
 		for (n = sn->children; n != NULL; n = n->next) {
-			gchar *key;
-			gchar *value;
+			const gchar *key;
+			const gchar *value;
 
-			key = (gchar*) n->data;
-			if (n->children)
-				value = (gchar*) n->children->data;
-			else
-				value = NULL;
+			key = as_yaml_node_get_key (n);
+			value = as_yaml_node_get_value (n);
 
 			if (g_strcmp0 (key, "unix-timestamp") == 0) {
 				as_release_set_timestamp (rel, g_ascii_strtoll (value, NULL, 10));
@@ -669,7 +696,7 @@ as_yamldata_process_releases (AsYAMLData *ydt, GNode *node, AsComponent *cpt)
 				as_release_set_description (rel, lvalue, NULL);
 				g_free (lvalue);
 			} else {
-				dep11_print_unknown ("release", key);
+				as_yaml_print_unknown ("release", key);
 			}
 		}
 
@@ -679,6 +706,80 @@ as_yamldata_process_releases (AsYAMLData *ydt, GNode *node, AsComponent *cpt)
 	}
 }
 
+/**
+ * as_yamldata_process_languages:
+ */
+static void
+as_yamldata_process_languages (GNode *node, AsComponent *cpt)
+{
+	GNode *sn;
+
+	for (sn = node->children; sn != NULL; sn = sn->next) {
+		GNode *n;
+		g_autofree gchar *locale = NULL;
+		g_autofree gchar *percentage_str = NULL;
+
+		for (n = sn->children; n != NULL; n = n->next) {
+			const gchar *key;
+			const gchar *value;
+
+			key = as_yaml_node_get_key (n);
+			value = as_yaml_node_get_value (n);
+
+			if (g_strcmp0 (key, "locale") == 0) {
+				if (locale == NULL)
+					locale = g_strdup (value);
+			} else if (g_strcmp0 (key, "percentage") == 0) {
+				if (percentage_str == NULL)
+					percentage_str = g_strdup (value);
+			} else {
+				as_yaml_print_unknown ("Languages", key);
+			}
+		}
+
+		if ((locale != NULL) && (percentage_str != NULL))
+			as_component_add_language (cpt,
+						   locale,
+						   g_ascii_strtoll (percentage_str, NULL, 10));
+	}
+}
+
+/**
+ * as_yamldata_process_suggests:
+ */
+static void
+as_yamldata_process_suggests (GNode *node, AsComponent *cpt)
+{
+	GNode *sn;
+
+	for (sn = node->children; sn != NULL; sn = sn->next) {
+		GNode *n;
+		g_autoptr(AsSuggested) sug = NULL;
+
+		sug = as_suggested_new ();
+		for (n = sn->children; n != NULL; n = n->next) {
+			const gchar *key;
+			const gchar *value;
+
+			key = as_yaml_node_get_key (n);
+			value = as_yaml_node_get_value (n);
+
+			if (g_strcmp0 (key, "type") == 0) {
+				as_suggested_set_kind (sug,
+						       as_suggested_kind_from_string (value));
+			} else if (g_strcmp0 (key, "ids") == 0) {
+				as_yaml_list_to_string_array (n,
+							      as_suggested_get_ids (sug));
+			} else {
+				as_yaml_print_unknown ("Suggests", key);
+			}
+		}
+
+		if (as_suggested_is_valid (sug))
+			as_component_add_suggested (cpt, sug);
+	}
+}
+
 /**
  * as_yamldata_process_component_node:
  */
@@ -705,21 +806,18 @@ as_yamldata_process_component_node (AsYAMLData *ydt, GNode *root)
 	as_component_set_priority (cpt, priv->default_priority);
 
 	for (node = root->children; node != NULL; node = node->next) {
-		gchar *key;
-		gchar *value;
+		const gchar *key;
+		const gchar *value;
 		gchar *lvalue;
 
 		if (node->children == NULL)
 			continue;
 
-		key = (gchar*) node->data;
-		value = (gchar*) node->children->data;
-		g_strstrip (value);
+		key = (const gchar*) node->data;
+		value = as_yaml_node_get_value (node);
 
 		if (g_strcmp0 (key, "Type") == 0) {
-			if (g_strcmp0 (value, "desktop-app") == 0)
-				as_component_set_kind (cpt, AS_COMPONENT_KIND_DESKTOP_APP);
-			else if (g_strcmp0 (value, "generic") == 0)
+			if (g_strcmp0 (value, "generic") == 0)
 				as_component_set_kind (cpt, AS_COMPONENT_KIND_GENERIC);
 			else
 				as_component_set_kind (cpt, as_component_kind_from_string (value));
@@ -727,6 +825,8 @@ as_yamldata_process_component_node (AsYAMLData *ydt, GNode *root)
 			as_component_set_id (cpt, value);
 		} else if (g_strcmp0 (key, "Priority") == 0) {
 			as_component_set_priority (cpt, g_ascii_strtoll (value, NULL, 10));
+		} else if (g_strcmp0 (key, "Merge") == 0) {
+			as_component_set_merge_kind (cpt, as_merge_kind_from_string (value));
 		} else if (g_strcmp0 (key, "Package") == 0) {
 			g_auto(GStrv) strv = NULL;
 			strv = g_new0 (gchar*, 1 + 1);
@@ -766,11 +866,11 @@ as_yamldata_process_component_node (AsYAMLData *ydt, GNode *root)
 		} else if (g_strcmp0 (key, "ProjectGroup") == 0) {
 			as_component_set_project_group (cpt, value);
 		} else if (g_strcmp0 (key, "Categories") == 0) {
-			dep11_list_to_string_array (node, categories);
+			as_yaml_list_to_string_array (node, categories);
 		} else if (g_strcmp0 (key, "CompulsoryForDesktops") == 0) {
-			dep11_list_to_string_array (node, compulsory_for_desktops);
+			as_yaml_list_to_string_array (node, compulsory_for_desktops);
 		} else if (g_strcmp0 (key, "Extends") == 0) {
-			dep11_list_to_string_array (node, as_component_get_extends (cpt));
+			as_yaml_list_to_string_array (node, as_component_get_extends (cpt));
 		} else if (g_strcmp0 (key, "Keywords") == 0) {
 			as_yamldata_process_keywords (ydt, node, cpt);
 		} else if (g_strcmp0 (key, "Url") == 0) {
@@ -781,10 +881,14 @@ as_yamldata_process_component_node (AsYAMLData *ydt, GNode *root)
 			dep11_process_provides (node, cpt);
 		} else if (g_strcmp0 (key, "Screenshots") == 0) {
 			as_yamldata_process_screenshots (ydt, node, cpt);
+		} else if (g_strcmp0 (key, "Languages") == 0) {
+			as_yamldata_process_languages (node, cpt);
 		} else if (g_strcmp0 (key, "Releases") == 0) {
 			as_yamldata_process_releases (ydt, node, cpt);
+		} else if (g_strcmp0 (key, "Suggests") == 0) {
+			as_yamldata_process_suggests (node, cpt);
 		} else {
-			dep11_print_unknown ("root", key);
+			as_yaml_print_unknown ("root", key);
 		}
 	}
 
@@ -874,13 +978,20 @@ as_yaml_emit_long_entry (yaml_emitter_t *emitter, const gchar *key, const gchar
 {
 	yaml_event_t event;
 	gint ret;
+	g_autofree gchar *value_clean = NULL;
 
 	if (value == NULL)
 		return;
 
 	as_yaml_emit_scalar_key (emitter, key);
-
-	yaml_scalar_event_initialize (&event, NULL, NULL, (yaml_char_t*) value, strlen (value), TRUE, TRUE, YAML_FOLDED_SCALAR_STYLE);
+	yaml_scalar_event_initialize (&event,
+					NULL,
+					NULL,
+					(yaml_char_t*) value,
+					strlen (value),
+					TRUE,
+					TRUE,
+					YAML_FOLDED_SCALAR_STYLE);
 	ret = yaml_emitter_emit (emitter, &event);
 	g_assert (ret);
 }
@@ -1086,7 +1197,7 @@ as_yaml_localized_list_helper (gchar *key, gchar **strv, yaml_emitter_t *emitter
 /**
  * as_yaml_emit_localized_lists:
  */
-void
+static void
 as_yaml_emit_localized_lists (yaml_emitter_t *emitter, const gchar *key, GHashTable *ltab)
 {
 	if (ltab == NULL)
@@ -1109,11 +1220,11 @@ as_yaml_emit_localized_lists (yaml_emitter_t *emitter, const gchar *key, GHashTa
 /**
  * as_yaml_emit_provides:
  */
-void
+static void
 as_yaml_emit_provides (yaml_emitter_t *emitter, AsComponent *cpt)
 {
 	GList *l;
-	GList *plist;
+	g_autoptr(GList) plist = NULL;
 	guint i;
 
 	g_auto(GStrv) dbus_system = NULL;
@@ -1281,7 +1392,7 @@ as_yaml_emit_provides (yaml_emitter_t *emitter, AsComponent *cpt)
 /**
  * as_yaml_emit_image:
  *
- * Helper for as_yaml_emit_screenshots
+ * Helper for as_yaml_data_emit_screenshots
  */
 static void
 as_yaml_emit_image (AsYAMLData *ydt, yaml_emitter_t *emitter, AsImage *img)
@@ -1308,14 +1419,15 @@ as_yaml_emit_image (AsYAMLData *ydt, yaml_emitter_t *emitter, AsImage *img)
 		as_yaml_emit_entry (emitter, "height", size);
 		g_free (size);
 	}
+	as_yaml_emit_entry (emitter, "lang", as_image_get_locale (img));
 	as_yaml_mapping_end (emitter);
 }
 
 /**
- * as_yaml_emit_screenshots:
+ * as_yaml_data_emit_screenshots:
  */
-void
-as_yaml_emit_screenshots (AsYAMLData *ydt, yaml_emitter_t *emitter, AsComponent *cpt)
+static void
+as_yaml_data_emit_screenshots (AsYAMLData *ydt, yaml_emitter_t *emitter, AsComponent *cpt)
 {
 	GPtrArray *sslist;
 	AsScreenshot *scr;
@@ -1369,6 +1481,9 @@ as_yaml_emit_screenshots (AsYAMLData *ydt, yaml_emitter_t *emitter, AsComponent
 
 }
 
+/**
+ * as_yaml_emit_icons:
+ */
 static void
 as_yaml_emit_icons (yaml_emitter_t *emitter, GPtrArray *icons)
 {
@@ -1450,12 +1565,174 @@ as_yaml_emit_icons (yaml_emitter_t *emitter, GPtrArray *icons)
 	}
 }
 
+/**
+ * as_yaml_emit_languages:
+ */
+static void
+as_yaml_emit_languages (yaml_emitter_t *emitter, AsComponent *cpt)
+{
+	GHashTableIter iter;
+	gpointer key, value;
+	GHashTable *lang_table;
+
+	lang_table = as_component_get_languages_map (cpt);
+	if (g_hash_table_size (lang_table) == 0)
+		return;
+
+	as_yaml_emit_scalar (emitter, "Languages");
+	as_yaml_sequence_start (emitter);
+
+	g_hash_table_iter_init (&iter, lang_table);
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		guint percentage;
+		const gchar *locale;
+		g_autofree gchar *percentage_str = NULL;
+
+		locale = (const gchar*) key;
+		percentage = GPOINTER_TO_INT (value);
+		percentage_str = g_strdup_printf("%i", percentage);
+
+		as_yaml_mapping_start (emitter);
+		as_yaml_emit_entry (emitter, "locale", locale);
+		as_yaml_emit_entry (emitter, "percentage", percentage_str);
+		as_yaml_mapping_end (emitter);
+	}
+
+	as_yaml_sequence_end (emitter);
+}
+
+/**
+ * as_yaml_emit_releases:
+ */
+static void
+as_yaml_data_emit_releases (AsYAMLData *ydt, yaml_emitter_t *emitter, AsComponent *cpt)
+{
+	guint i;
+	GPtrArray *releases;
+	AsYAMLDataPrivate *priv = GET_PRIVATE (ydt);
+
+	releases = as_component_get_releases (cpt);
+	if (releases->len == 0)
+		return;
+
+	as_yaml_emit_scalar (emitter, "Releases");
+	as_yaml_sequence_start (emitter);
+
+	for (i = 0; i < releases->len; i++) {
+		guint j;
+		guint64 unixtime;
+		AsUrgencyKind urgency;
+		GPtrArray *locations;
+		AsRelease *rel = AS_RELEASE (g_ptr_array_index (releases, i));
+
+		/* start mapping for this release */
+		as_yaml_mapping_start (emitter);
+
+		/* version */
+		as_yaml_emit_entry (emitter,
+				    "version",
+				    as_release_get_version (rel));
+
+		/* timestamp & date */
+		unixtime = as_release_get_timestamp (rel);
+		if (unixtime > 0) {
+			g_autofree gchar *time_str = NULL;
+
+			if (priv->mode == AS_PARSER_MODE_DISTRO) {
+				time_str = g_strdup_printf ("%ld", unixtime);
+				as_yaml_emit_entry (emitter, "unix-timestamp", time_str);
+			} else {
+				GTimeVal time;
+				time.tv_sec = unixtime;
+				time.tv_usec = 0;
+				time_str = g_time_val_to_iso8601 (&time);
+				as_yaml_emit_entry (emitter, "date", time_str);
+			}
+		}
+
+		/* urgency */
+		urgency = as_release_get_urgency (rel);
+		if (urgency != AS_URGENCY_KIND_UNKNOWN) {
+			as_yaml_emit_entry (emitter,
+					    "urgency",
+					    as_urgency_kind_to_string (urgency));
+		}
+
+		/* description */
+		as_yaml_emit_long_localized_entry (emitter,
+						   "description",
+						   as_release_get_description_table (rel));
+
+		/* location URLs */
+		locations = as_release_get_locations (rel);
+		if (locations->len > 0) {
+			as_yaml_emit_scalar (emitter, "locations");
+			as_yaml_sequence_start (emitter);
+			for (j = 0; j < locations->len; j++) {
+				const gchar *lurl;
+				lurl = (const gchar*) g_ptr_array_index (locations, j);
+				as_yaml_emit_scalar (emitter, lurl);
+			}
+
+			as_yaml_sequence_end (emitter);
+		}
+
+		/* TODO: Checksum and size are missing, because they are not specified yet for DEP-11.
+		 * Will need to be added when/if we need it. */
+
+		/* end mapping for the release */
+		as_yaml_mapping_end (emitter);
+	}
+
+	as_yaml_sequence_end (emitter);
+}
+
+/**
+ * as_yaml_data_emit_suggests:
+ */
+static void
+as_yaml_data_emit_suggests (AsYAMLData *ydt, yaml_emitter_t *emitter, AsComponent *cpt)
+{
+	guint i;
+	GPtrArray *suggestions;
+
+	suggestions = as_component_get_suggested (cpt);
+	if (suggestions->len == 0)
+		return;
+
+	as_yaml_emit_scalar (emitter, "Suggested");
+	as_yaml_sequence_start (emitter);
+
+	for (i = 0; i < suggestions->len; i++) {
+		AsSuggested *sug = AS_SUGGESTED (g_ptr_array_index (suggestions, i));
+
+		/* start mapping for this suggestion */
+		as_yaml_mapping_start (emitter);
+
+		/* type */
+		as_yaml_emit_entry (emitter,
+				    "type",
+				    as_suggested_kind_to_string (as_suggested_get_kind (sug)));
+
+		/* component-ids */
+		as_yaml_emit_sequence (emitter,
+				       "ids",
+					as_suggested_get_ids (sug));
+
+		/* end mapping for the suggestion list */
+		as_yaml_mapping_end (emitter);
+	}
+
+	as_yaml_sequence_end (emitter);
+}
+
 /**
  * as_yaml_serialize_component:
  */
 static void
 as_yaml_serialize_component (AsYAMLData *ydt, yaml_emitter_t *emitter, AsComponent *cpt)
 {
+	AsYAMLDataPrivate *priv = GET_PRIVATE (ydt);
 	guint i;
 	gint res;
 	const gchar *cstr;
@@ -1466,7 +1743,7 @@ as_yaml_serialize_component (AsYAMLData *ydt, yaml_emitter_t *emitter, AsCompone
 	GPtrArray *icons;
 
 	/* we only serialize a component with minimal necessary information */
-	if (!as_component_is_valid (cpt)) {
+	if ((priv->check_valid) && (!as_component_is_valid (cpt))) {
 		g_debug ("Can not serialize '%s': Component is invalid.", as_component_get_id (cpt));
 		return;
 	}
@@ -1492,6 +1769,21 @@ as_yaml_serialize_component (AsYAMLData *ydt, yaml_emitter_t *emitter, AsCompone
 	/* AppStream-ID */
 	as_yaml_emit_entry (emitter, "ID", as_component_get_id (cpt));
 
+	/* priority */
+	if (as_component_get_priority (cpt) != 0) {
+		g_autofree gchar *priority_str = g_strdup_printf ("%i", as_component_get_priority (cpt));
+		as_yaml_emit_entry (emitter,
+				    "Priority",
+				    priority_str);
+	}
+
+	/* merge strategy */
+	if (as_component_get_merge_kind (cpt) != AS_MERGE_KIND_NONE) {
+		as_yaml_emit_entry (emitter,
+				    "Merge",
+				    as_merge_kind_to_string (as_component_get_merge_kind (cpt)));
+	}
+
 	/* SourcePackage */
 	as_yaml_emit_entry (emitter, "SourcePackage", as_component_get_source_pkgname (cpt));
 
@@ -1593,9 +1885,16 @@ as_yaml_serialize_component (AsYAMLData *ydt, yaml_emitter_t *emitter, AsCompone
 	as_yaml_emit_provides (emitter, cpt);
 
 	/* Screenshots */
-	as_yaml_emit_screenshots (ydt, emitter, cpt);
+	as_yaml_data_emit_screenshots (ydt, emitter, cpt);
 
-	/* TODO: Releases, Translations */
+	/* Translation details */
+	as_yaml_emit_languages (emitter, cpt);
+
+	/* Releases */
+	as_yaml_data_emit_releases (ydt, emitter, cpt);
+
+	/* Suggests */
+	as_yaml_data_emit_suggests (ydt, emitter, cpt);
 
 	/* close main mapping */
 	as_yaml_mapping_end (emitter);
@@ -1614,9 +1913,9 @@ as_yaml_serialize_component (AsYAMLData *ydt, yaml_emitter_t *emitter, AsCompone
 static void
 as_yamldata_write_header (AsYAMLData *ydt, yaml_emitter_t *emitter)
 {
+	AsYAMLDataPrivate *priv = GET_PRIVATE (ydt);
 	gint res;
 	yaml_event_t event;
-	AsYAMLDataPrivate *priv = GET_PRIVATE (ydt);
 
 	yaml_document_start_event_initialize (&event, NULL, NULL, NULL, FALSE);
 	res = yaml_emitter_emit (emitter, &event);
@@ -1740,7 +2039,8 @@ as_yamldata_parse_distro_data (AsYAMLData *ydt, const gchar *data, GError **erro
 	yaml_event_t event;
 	gboolean header = TRUE;
 	gboolean parse = TRUE;
-	GPtrArray *cpts = NULL;;
+	gboolean ret = TRUE;
+	g_autoptr(GPtrArray) cpts = NULL;
 	AsYAMLDataPrivate *priv = GET_PRIVATE (ydt);
 
 	/* we ignore empty data - usually happens if the file is broken, e.g. by disk corruption
@@ -1768,16 +2068,35 @@ as_yamldata_parse_distro_data (AsYAMLData *ydt, const gchar *data, GError **erro
 	yaml_parser_set_input_string (&parser, (unsigned char*) data, strlen (data));
 
 	while (parse) {
-		yaml_parser_parse (&parser, &event);
+		if (!yaml_parser_parse (&parser, &event)) {
+			g_set_error (error,
+					AS_METADATA_ERROR,
+					AS_METADATA_ERROR_PARSE,
+					"Invalid DEP-11 file found. Could not parse YAML: %s", parser.problem);
+			ret = FALSE;
+			break;
+		}
+
 		if (event.type == YAML_DOCUMENT_START_EVENT) {
 			GNode *n;
-			gchar *key;
-			gchar *value;
+			const gchar *key;
+			const gchar *value;
 			AsComponent *cpt;
 			gboolean header_found = FALSE;
-			GNode *root = g_node_new (g_strdup (""));
+			GError *tmp_error = NULL;
+			g_autoptr(GNode) root = NULL;
 
-			dep11_yaml_process_layer (&parser, root);
+			root = g_node_new (g_strdup (""));
+			as_yaml_process_layer (&parser, root, &tmp_error);
+			if (tmp_error != NULL) {
+				/* stop immediately, since we found an error when parsing the document */
+				g_propagate_error (error, tmp_error);
+				g_free (root->data);
+				yaml_event_delete (&event);
+				ret = FALSE;
+				parse = FALSE;
+				break;
+			}
 
 			if (header) {
 				for (n = root->children; n != NULL; n = n->next) {
@@ -1787,11 +2106,12 @@ as_yamldata_parse_distro_data (AsYAMLData *ydt, const gchar *data, GError **erro
 								AS_METADATA_ERROR,
 								AS_METADATA_ERROR_FAILED,
 								"Invalid DEP-11 file found: Header invalid");
+						ret = FALSE;
 						break;
 					}
 
-					key = (gchar*) n->data;
-					value = (gchar*) n->children->data;
+					key = as_yaml_node_get_key (n);
+					value = as_yaml_node_get_value (n);
 
 					if (g_strcmp0 (key, "File") == 0) {
 						if (g_strcmp0 (value, "DEP-11") != 0) {
@@ -1839,6 +2159,7 @@ as_yamldata_parse_distro_data (AsYAMLData *ydt, const gchar *data, GError **erro
 				if (cpt == NULL) {
 					g_warning ("Parsing of YAML metadata failed: Could not read data for component.");
 					parse = FALSE;
+					ret = FALSE;
 				}
 
 				/* add found component to the results set */
@@ -1849,21 +2170,24 @@ as_yamldata_parse_distro_data (AsYAMLData *ydt, const gchar *data, GError **erro
 					G_IN_ORDER,
 					G_TRAVERSE_ALL,
 					-1,
-					as_yamldata_free_node,
+					as_yaml_free_node,
 					NULL);
-			g_node_destroy (root);
 		}
 
 		/* stop if end of stream is reached */
 		if (event.type == YAML_STREAM_END_EVENT)
 			parse = FALSE;
 
-		yaml_event_delete(&event);
+		yaml_event_delete (&event);
 	}
 
 	yaml_parser_delete (&parser);
 
-	return cpts;
+	/* return NULL on error, otherwise return the list of found components */
+	if (ret)
+		return g_ptr_array_ref (cpts);
+	else
+		return NULL;
 }
 
 /**
@@ -1879,16 +2203,9 @@ as_yamldata_parse_distro_data (AsYAMLData *ydt, const gchar *data, GError **erro
 void
 as_yamldata_set_locale (AsYAMLData *ydt, const gchar *locale)
 {
-	gchar **strv;
 	AsYAMLDataPrivate *priv = GET_PRIVATE (ydt);
-
 	g_free (priv->locale);
-	g_free (priv->locale_short);
 	priv->locale = g_strdup (locale);
-
-	strv = g_strsplit (priv->locale, "_", 0);
-	priv->locale_short = g_strdup (strv[0]);
-	g_strfreev (strv);
 }
 
 /**
@@ -1900,13 +2217,27 @@ as_yamldata_set_locale (AsYAMLData *ydt, const gchar *locale)
  *
  * Returns: Locale used for metadata parsing.
  **/
-const gchar *
+const gchar*
 as_yamldata_get_locale (AsYAMLData *ydt)
 {
 	AsYAMLDataPrivate *priv = GET_PRIVATE (ydt);
 	return priv->locale;
 }
 
+/**
+ * as_yamldata_set_check_valid:
+ * @check: %TRUE if check should be enabled.
+ *
+ * Set whether we should perform basic validity checks on the component
+ * before serializing it to YAML.
+ */
+void
+as_yamldata_set_check_valid (AsYAMLData *ydt, gboolean check)
+{
+	AsYAMLDataPrivate *priv = GET_PRIVATE (ydt);
+	priv->check_valid = check;
+}
+
 /**
  * as_yamldata_class_init:
  **/
