@@ -36,14 +36,17 @@ int
 ascli_show_status (void)
 {
 	guint i;
-	g_autoptr(AsDatabase) cache = NULL;
+	g_autoptr(AsDataPool) dpool = NULL;
+	g_autoptr(GError) error = NULL;
 	const gchar *metainfo_path = "/usr/share/metainfo";
 	const gchar *appdata_path = "/usr/share/appdata";
 
+	/* TRANSLATORS: In the status report of ascli: Header */
 	ascli_print_highlight (_("AppStream Status:"));
-	ascli_print_stdout ("Version: %s", VERSION);
+	ascli_print_stdout (_("Version: %s"), VERSION);
 	g_print ("\n");
 
+	/* TRANSLATORS: In the status report of ascli: Refers to the metadata shipped by distributions */
 	ascli_print_highlight (_("Distribution metadata:"));
 	for (i = 0; AS_APPSTREAM_METADATA_PATHS[i] != NULL; i++) {
 		g_autofree gchar *xml_path = NULL;
@@ -107,7 +110,8 @@ ascli_show_status (void)
 		g_print ("\n");
 	}
 
-	ascli_print_highlight (_("Upstream metadata:"));
+	/* TRANSLATORS: Info about upstream metadata / metainfo files in the ascli status report */
+	ascli_print_highlight (_("Metainfo files:"));
 	if (g_file_test (metainfo_path, G_FILE_TEST_IS_DIR)) {
 		g_autoptr(GPtrArray) xmls = NULL;
 		g_autofree gchar *msg = NULL;
@@ -119,6 +123,7 @@ ascli_show_status (void)
 		}
 	} else {
 		if (!g_file_test (appdata_path, G_FILE_TEST_IS_DIR))
+			/* TRANSLATORS: No metainfo files have been found */
 			ascli_print_stdout ("  - %s", _("Empty."));
 	}
 	if (g_file_test (appdata_path, G_FILE_TEST_IS_DIR)) {
@@ -127,28 +132,29 @@ ascli_show_status (void)
 
 		xmls = as_utils_find_files_matching (appdata_path, "*.xml", FALSE, NULL);
 		if (xmls != NULL) {
+			/* TRANSLATORS: Found metainfo files in legacy directories */
 			msg = g_strdup_printf (_("Found %i components in legacy paths."), xmls->len);
 			ascli_print_stdout ("  - %s", msg);
 		}
 	}
 	g_print ("\n");
 
+	/* TRANSLATORS: Status summary in ascli */
 	ascli_print_highlight (_("Summary:"));
-	cache = as_database_new ();
-	if (g_file_test (as_database_get_location (cache), G_FILE_TEST_IS_DIR)) {
-		g_autoptr(GPtrArray) cpts = NULL;
-		g_autoptr(GError) error = NULL;
 
-		ascli_print_stdout (_("The system metadata cache exists."));
-		as_database_open (cache, NULL);
-		cpts = as_database_get_all_components (cache, &error);
-		if (error != NULL) {
-			ascli_print_stdout (_("There was an error while trying to find additional information: %s"), error->message);
-		} else {
-			ascli_print_stdout (_("We have information on %i software components."), cpts->len);
-		}
+	dpool = as_data_pool_new ();
+	as_data_pool_load (dpool, NULL, &error);
+	if (error == NULL) {
+		g_autoptr(GPtrArray) cpts = NULL;
+		cpts = as_data_pool_get_components (dpool);
+
+		ascli_print_stdout (_("We have information on %i software components."), cpts->len);
+		/* TODO: Request the on-disk cache status from #AsDataPool and display it here.
+		 * ascli_print_stdout (_("The system metadata cache exists."));
+		 * ascli_print_stdout (_("The system metadata cache does not exist."));
+		 */
 	} else {
-		ascli_print_stdout (_("The system metadata cache does not exist."));
+		ascli_print_stderr (_("Error while loading the metadata pool: %s"), error->message);
 	}
 
 	return 0;
