@@ -25,6 +25,8 @@
 #include <glib/gi18n-lib.h>
 #include <appstream.h>
 
+#include "ascli-utils.h"
+
 /**
  * importance_to_print_string:
  **/
@@ -80,10 +82,13 @@ process_report (GList *issues, gboolean pretty, gboolean pedantic)
 	AsValidatorIssue *issue;
 	AsIssueImportance importance;
 	gboolean no_errors = TRUE;
-	gchar *header;
 
 	for (l = issues; l != NULL; l = l->next) {
-		issue = (AsValidatorIssue*) l->data;
+		g_autofree gchar *location = NULL;
+		g_autofree gchar *header = NULL;
+		g_autofree gchar *message = NULL;
+
+		issue = AS_VALIDATOR_ISSUE (l->data);
 		importance = as_validator_issue_get_importance (issue);
 
 		/* if there are errors or warnings, we consider the validation to be failed */
@@ -94,13 +99,15 @@ process_report (GList *issues, gboolean pretty, gboolean pedantic)
 		if ((!pedantic) && (importance == AS_ISSUE_IMPORTANCE_PEDANTIC))
 			continue;
 
+		location = as_validator_issue_get_location (issue);
 		header = importance_location_to_print_string (importance,
-								as_validator_issue_get_location (issue),
+								location,
 								pretty);
+
+		message = ascli_format_long_output (as_validator_issue_get_message (issue), 4);
 		g_print ("%s\n    %s\n\n",
 				header,
-				as_validator_issue_get_message (issue));
-		g_free (header);
+				message);
 	}
 
 	return no_errors;
